function pk = slice_points_interpolation(p1,p2,n,m,flag_fig) 
% SLICE_POINTS_INTERPOLATION takes two sets of points and the range of 
% interpolition n & m and return pk which is the interpolants points 
% [n:(n-m+1):m] 
% 
% the sampling is done by computing
%  v = p1-p2
% pk = p2 + d/norm(v)*v where d is the increment of the distances
%
% written by bepi@dtu.dk first version 2019.12.02

%default no figure
if nargout < 3
    flag_fig = 0;
end

d_p = sqrt(sum((p1-p2).^2,2));

pk = nan(n-m+1,3,length(d_p));
for j = 1:length(d_p)
    sim = linspace(0,d_p(j),n-m+1);
    for i = 1:length(sim)
        if sim(i) == 0
            pk(i,1:2,j) = p2(j,:);
        else
            pk(i,1:2,j) = p2(j,:) + (sim(i)/d_p(j)).*(p1(j,:)-p2(j,:));
        end
        pk(i,3,j) = m + i-1;%n,m = (220:800)';
    end
end

if flag_fig 
    figure,
    plot3([p2(:,1),p2(:,1)]',...
          [p1(:,2),p1(:,2)]',...
          [ones(size(p1,1),1)*n,ones(size(p1,1),1)*m]')
    xlabel('x') ; ylabel('y') ; zlabel('z')
    hold on

    for s = 1:size(pk,3)
        tmp = squeeze(pk(:,:,s));
        plot3(tmp(:,1),tmp(:,2),tmp(:,3),'-xm')
    end
end

end


